/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Buttons.c,v 1.3 1994/02/18 19:56:12 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Buttons.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Rick Odermatt
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/BulletinB.h>
#include <Xm/BulletinBP.h>

#include "Xn.h"

#define WIDTH (100 * 20)

static void     button_resize (XmBulletinBoardWidget bw, XtPointer nu1, XtPointer nu2)
{
    Dimension       w, bbw, *buttonWidths, totalButtonWidth;
    int             xpos, buttonGap;
    unsigned char   unitType;
    int             i;

    Widget         *childList = bw->composite.children;
    int             numChildren = bw->composite.num_children;
    int             numManaged = 0;

    if (numChildren == 0)
	return;

    XtVaGetValues ((Widget) bw,
	XmNwidth, &w,
	XmNunitType, &unitType,
	NULL);

    bbw = w;

    buttonWidths = (Dimension *) XtMalloc (numChildren * sizeof (Dimension));
    totalButtonWidth = 0;
    for (i = 0; i < numChildren; i++) {
	if (XtIsManaged (childList[i])) {
	    XtVaGetValues ((Widget) bw,
		XmNwidth, &w,
		NULL);

	    buttonWidths[i] = w;
	    totalButtonWidth += w;
	    numManaged++;
	}
    }

    if (bbw < totalButtonWidth) {
	bbw = totalButtonWidth;
	XtVaSetValues ((Widget) bw,
	    XmNwidth, bbw,
	    NULL);
    }
    if (numManaged == 1) {
	xpos = (bbw - totalButtonWidth) / 2;
    } else {
	xpos = 0;
	buttonGap = (bbw - totalButtonWidth) / (numManaged - 1);
    }

    for (i = 0; i < numChildren; i++) {
	if (XtIsManaged (childList[i])) {
	    w = XmConvertUnits ((Widget) bw, XmHORIZONTAL, unitType, xpos, XmPIXELS);
	    XtMoveWidget (childList[i], w, 0);
	    xpos += buttonGap + buttonWidths[i];
	}
    }

    XtFree ((XtPointer) buttonWidths);
}

Widget          XnCreateButtonHeader (Widget parent, XnCallbackData *data, int *tabs, int tabcount)
{
    int             i, offset;
    Widget          header;

    header = XtVaCreateWidget ("labelForm",
	    xmFormWidgetClass, parent,
	    XmNunitType, Xm100TH_FONT_UNITS,
	    NULL);

    if (XmIsForm (parent)) {
	XtVaSetValues (header,
		       XmNleftAttachment, XmATTACH_FORM,
		       NULL);
    }

    for (offset = i = 0; data[i].name; i++) {
	data [i].w = XtVaCreateManagedWidget (data[i].name,
		xmPushButtonWidgetClass, header,
		XmNleftAttachment, XmATTACH_FORM,
		XmNtopAttachment, XmATTACH_FORM,
		XmNshadowThickness, 0,
		XmNleftOffset, offset * 100,
		NULL);
	if (data[i].proc)
	    XnAddCallback (data[i].w, XmNactivateCallback, data[i].proc, data[i].data);
	XtVaSetValues (data[i].w, XmNuserData, i, NULL);
	offset += strlen (data[i].name);
	offset += (offset + 8) % 8;
    }
    XtManageChild (header);
    return header;
}

Widget          XnCreateLabelHeader (Widget parent, XnCallbackData *data, int *tabs, int tabcount)
{
    int             i, offset;
    Widget          header;

    header = XtVaCreateWidget ("labelForm",
	    xmFormWidgetClass, parent,
	    XmNunitType, Xm100TH_FONT_UNITS,
	    NULL);

    if (XmIsForm (parent)) {
	XtVaSetValues (header,
		       XmNleftAttachment, XmATTACH_FORM,
		       NULL);
    }

    for (offset = i = 0; data[i].name; i++) {
	data [i].w = XtVaCreateManagedWidget (data[i].name,
		xmLabelWidgetClass, header,
		XmNleftAttachment, XmATTACH_FORM,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftOffset, offset * 100,
		NULL);
	offset += strlen (data[i].name);
	offset += (offset + 8) % 8;
    }
    XtManageChild (header);
    return header;
}

Widget          XnCreateButtonFooter (Widget parent, XnCallbackData *data)
{
    int             i;
    Widget          footer;

    footer = XmCreateBulletinBoard (parent, "buttonForm", NULL, 0);

    XtAddEventHandler (footer, StructureNotifyMask, FALSE,
	(XtEventHandler) button_resize, NULL);

    for (i = 0; data [i].name; i++) {
	data[i].w = XtVaCreateManagedWidget (data[i].name,
					     xmPushButtonWidgetClass, footer,
					     XmNunitType, Xm100TH_FONT_UNITS,
					     XmNwidth, WIDTH,
					     XmNx, 0,
					     XmNy, 0,
					     NULL);
	if (data[i].proc)
	    XnAddCallback (data[i].w, XmNactivateCallback, data[i].proc, data[i].data);
    }

    if (XmIsForm (parent)) {
	XtVaSetValues (footer,
		       XmNleftAttachment, XmATTACH_FORM,
		       XmNrightAttachment, XmATTACH_FORM,
		       XmNbottomAttachment, XmATTACH_FORM,
		       NULL);
    }
    XtManageChild (footer);
    return (footer);
}

Widget         XnCreateLabelFooter (Widget parent, XnCallbackData *data)
{
    int             i;
    Widget          footer;

    footer = XmCreateBulletinBoard (parent, "buttonForm", NULL, 0);

    XtVaSetValues (footer,
	XmNmarginHeight, 0,
	XmNmarginWidth, 0,
	NULL);

    XtAddEventHandler (footer, StructureNotifyMask, FALSE,
	(XtEventHandler) button_resize, NULL);

    for (i = 0; data [i].name; i++) {
	data[i].w = XtVaCreateManagedWidget (data[i].name,
					     xmLabelWidgetClass, footer,
					     XmNunitType, Xm100TH_FONT_UNITS,
					     XmNwidth, WIDTH,
					     XmNx, 0,
					     XmNy, 0,
					     NULL);
    }

    if (XmIsForm (parent)) {
	XtVaSetValues (footer,
		       XmNleftAttachment, XmATTACH_FORM,
		       XmNrightAttachment, XmATTACH_FORM,
		       XmNbottomAttachment, XmATTACH_FORM,
		       NULL);
    }
    XtManageChild (footer);
    return (footer);
}
