/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Color.c,v 1.6 1994/02/13 06:42:41 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Color.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <assert.h>
#include <malloc.h>
#include <math.h>

#include <X11/Intrinsic.h>

#include <Xn/Xn.h>

#define min(x, y)	((x < y) ? x : y)
#define max(x, y)	((x > y) ? x : y)

#define MAX_INTENSITY	65535.0

void            XnXColortoHSV (hsv, red, green, blue)
XnHSVColor     *hsv;
int             red;
int             green;
int             blue;
{
    double          rbg_min, rgb_max;
    double          delta;
    double          r;
    double          g;
    double          b;

    r = (double) (red) / MAX_INTENSITY;
    g = (double) (green) / MAX_INTENSITY;
    b = (double) (blue) / MAX_INTENSITY;

    rgb_max = max (max (r, g), b);
    rbg_min = min (min (r, g), b);

    hsv->v = rgb_max;

    if (rgb_max != 0.0)
	hsv->s = (rgb_max - rbg_min) / rgb_max;
    else
	hsv->s = 0.0;
    if (hsv->s != 0.0) {
	delta = rgb_max - rbg_min;
	if (r == rgb_max)
	    hsv->h = (g - b) / delta;
	else if (g == rgb_max)
	    hsv->h = 2.0 + (b - r) / delta;
	else if (b == rgb_max)
	    hsv->h = 4.0 + (r - g) / delta;
	hsv->h *= 60.0;
	if (hsv->h < 0.0)
	    hsv->h += 360.0;
    } else
	hsv->h = 0.0;
}

void            XnHSVtoXColor (c, h, s, v)
XColor         *c;
double          h;
double          s;
double          v;
{
    double          f, p, q, t, r, g, b;
    int             i;

    if (s == 0.0) {
	r = v;
	g = v;
	b = v;
    } else {
	while (h >= 360.0)
	    h -= 360.0;
	while (h < 0.0)
	    h += 360.0;

	h /= 60.0;
	i = (int) (h);
	f = h - (double) (i);
	p = v * (1.0 - s);
	q = v * (1.0 - (s * f));
	t = v * (1.0 - (s * (1.0 - f)));
	switch (i) {
	    case 0:
		r = v;
		g = t;
		b = p;
		break;
	    case 1:
		r = q;
		g = v;
		b = p;
		break;
	    case 2:
		r = p;
		g = v;
		b = t;
		break;
	    case 3:
		r = p;
		g = q;
		b = v;
		break;
	    case 4:
		r = t;
		g = p;
		b = v;
		break;
	    case 5:
		r = v;
		g = p;
		b = q;
		break;
	    default:
		r = g = b = 0.0;
	}
    }
    c->flags = DoRed | DoGreen | DoBlue;
    c->red = (unsigned short) (r * MAX_INTENSITY);
    c->green = (unsigned short) (g * MAX_INTENSITY);
    c->blue = (unsigned short) (b * MAX_INTENSITY);
}

void            XnHSVInitBand (band, beg, end)
XnHSVBand      *band;
XnHSVColor     *beg;
XnHSVColor     *end;
{
    double          h, hd;
    double          s, sd;
    double          v, vd;
    int             i;

    assert (band);

    band->beg = *beg;
    band->end = *end;

    if (band->count > 2) {
	hd = (end->h - beg->h) / (band->count - 1);
	sd = (end->s - beg->s) / (band->count - 1);
	vd = (end->v - beg->v) / (band->count - 1);

	for (i = 0; i < band->count; i++) {
	    h = beg->h + (double) (i * hd);
	    s = beg->s + (double) (i * sd);
	    v = beg->v + (double) (i * vd);
	    XnHSVtoXColor (band->colors + i, h, s, v);
	}
    } else if (band->count == 2) {
	h = beg->h + (end->h - beg->h) / 2;
	s = beg->s + (end->s - beg->s) / 2;
	v = beg->v + (end->v - beg->v) / 2;
	XnHSVtoXColor (band->colors + i, h, s, v);
    } else if (band->count == 1) {
	XnHSVtoXColor (band->colors, beg->h, beg->s, beg->v);
    }
}

XnHSVBand      *XnHSVNewBand (count)
int             count;
{
    XnHSVBand      *band;

    if ((band = (XnHSVBand *) malloc (sizeof (XnHSVBand)))) {
	band->count = count;
	if (!(band->colors = (XColor *) malloc (sizeof (XColor) * count))) {
	    free (band);
	    band = NULL;
	} else {
	    band->beg.h = 0.0;
	    band->beg.s = 1.0;
	    band->beg.v = 1.0;
	    band->end.h = 360.0;
	    band->end.s = 1.0;
	    band->end.v = 1.0;
	    XnHSVInitBand (band, &band->beg, &band->end);
	}
    }
    return (band);
}

void            XnHSVDeleteBand (line)
XnHSVBand      *line;
{
    assert (line);
    assert (line->colors);
    free (line->colors);
    free (line);
}

XnHSVGrid      *XnNewHSVGrid (rows, cols)
int             rows;
int             cols;
{
    XnHSVGrid      *grid;
    int             band;

    if ((grid = (XnHSVGrid *) malloc (sizeof (XnHSVGrid)))) {
	grid->cols = cols;
	if (!(grid->bands = (XnHSVBand **)
	      malloc (sizeof (XnHSVBand *) * rows))) {
	    free (grid);
	    grid = NULL;
	}
	grid->cols = cols;
	grid->rows = rows;
	for (band = 0; band < rows; band++) {
	    if (!(grid->bands[band] = XnHSVNewBand (cols))) {
		while (--band >= 0) {
		    XnHSVDeleteBand (grid->bands[band]);
		}
		free (grid);
		grid = NULL;
		break;
	    }
	}
    }
    return (grid);
}

void            XnDeleteHSVGrid (grid)
XnHSVGrid      *grid;
{
    int             band;

    assert (grid);
    assert (grid->bands);

    for (band = 0; band < grid->rows; band++) {
	XnHSVDeleteBand (grid->bands[band]);
    }
    free (grid);
}

void            XnHSVInitGrid (grid, ul, ur, ll, lr)
XnHSVGrid      *grid;
XnHSVColor     *ul;
XnHSVColor     *ur;
XnHSVColor     *ll;
XnHSVColor     *lr;
{
    double          dh1;
    double          ds1;
    double          dv1;
    double          dh2;
    double          ds2;
    double          dv2;
    int             rows;
    int             band;

    assert (grid);

    if (&grid->bands [0]->beg != ul)
	grid->bands [0]->beg = *ul;

    if (&grid->bands [0]->end != ur)
	grid->bands [0]->end = *ur;

    if (&grid->bands [grid->rows - 1]->beg != ll)
	grid->bands [grid->rows - 1]->beg = *ll;

    if (&grid->bands [grid->rows - 1]->end != lr)
	grid->bands [grid->rows - 1]->end = *lr;

    if (grid->rows > 2) {
	rows = grid->rows - 1;

	dh1 = (grid->bands[rows]->beg.h - grid->bands[0]->beg.h) / rows;
	ds1 = (grid->bands[rows]->beg.s - grid->bands[0]->beg.s) / rows;
	dv1 = (grid->bands[rows]->beg.v - grid->bands[0]->beg.v) / rows;

	dh2 = (grid->bands[rows]->end.h - grid->bands[0]->end.h) / rows;
	ds2 = (grid->bands[rows]->end.s - grid->bands[0]->end.s) / rows;
	dv2 = (grid->bands[rows]->end.v - grid->bands[0]->end.v) / rows;
    }
    for (band = 0; band < grid->rows; band++) {
	if (band > 0 && band < grid->rows - 1) {
	    grid->bands[band]->beg.h = grid->bands[0]->beg.h + band * dh1;
	    grid->bands[band]->beg.s = grid->bands[0]->beg.s + band * ds1;
	    grid->bands[band]->beg.v = grid->bands[0]->beg.v + band * dv1;

	    grid->bands[band]->end.h = grid->bands[0]->end.h + band * dh2;
	    grid->bands[band]->end.s = grid->bands[0]->end.s + band * ds2;
	    grid->bands[band]->end.v = grid->bands[0]->end.v + band * dv2;
	}
	XnHSVInitBand (grid->bands[band],
		       &grid->bands[band]->beg, &grid->bands[band]->end);
    }
}

void            XnHSVBandResize (band, count)
XnHSVBand      *band;
int             count;
{
    band->colors = (XColor *) realloc ((char *) band->colors,
				       sizeof (XColor) * count);
    band->count = count;
}

int             XnHSVGridResize (grid, rows, cols)
XnHSVGrid      *grid;
int             rows;
int             cols;
{
    XnHSVBand     **tmp;
    int             i;

    assert (rows > 0);
    assert (cols > 0);

    /*
     * Copy the last band into the future last band
     */

    if (grid->rows > 2 && grid->rows > rows) {
	XnHSVDeleteBand (grid->bands[rows - 1]);
	grid->bands[rows - 1] = grid->bands[grid->rows - 1];
    }

    /*
     * Clear out the rest of the unused bands and realloc
     */

    for (i = rows; i < grid->rows - 1; i++) {
	XnHSVDeleteBand (grid->bands[rows - 1]);
    }

    tmp = (XnHSVBand **) realloc ((char *) grid->bands,
				  sizeof (XnHSVBand *) * rows);

    grid->bands = tmp;

    /*
     * if new bands have been added copy the old last band to the end
     */

    if (grid->rows > 2 && grid->rows < rows) {
	grid->bands[rows - 1] = grid->bands[grid->rows - 1];
    }

    /*
     * init all of the new bands
     */

    for (i = 0; i < rows; i++) {
	if (i >= grid->rows - 1 || i < rows - 1) {
	    grid->bands[i] = XnHSVNewBand (cols);
	}
	if (cols != grid->bands[0]->count)
	    XnHSVBandResize (grid->bands[i], cols);
    }
    grid->cols = cols;
    grid->rows = rows;
    XnHSVInitGrid (grid,
		   &grid->bands[0]->beg,
		   &grid->bands[0]->end,
		   &grid->bands[grid->rows - 1]->beg,
		   &grid->bands[grid->rows - 1]->end);
    return (0);
}

void            XnHSVRenderGrid (grid, display, pixmap, gc, width, height)
XnHSVGrid      *grid;
Display        *display;
Pixmap          pixmap;
GC              gc;
int             width;
int             height;
{
    double          dx;
    double          dy;
    int             x;
    int             y;

    dy = (double) (height) / (double) (grid->rows);
    dx = (double) (width) / (double) (grid->cols);

    for (y = 0; y < grid->rows; y++) {
	for (x = 0; x < grid->cols; x++) {
	    XSetForeground (display, gc, grid->bands[y]->colors[x].pixel);
	    XFillRectangle (display, pixmap, gc,
	    (int) (x * dx), (int) (y * dy), (int) (dx) + 1, (int) (dy) + 1);
	}
    }
}

XnColormapData *XnNewColormapData (display)
Display        *display;
{
    XnColormapData *data;

    data = (XnColormapData *) malloc (sizeof (XnColormapData));
    data->cmap = DefaultColormap (display, DefaultScreen (display));
    data->pixels = NULL;
    data->count = 0;
    return (data);
}

int             XnAllocColors (display, m, count)
Display        *display;
XnColormapData *m;
int             count;
{
    XColor          rgb;
    int             i;

    if (count == m->count)
	return (m->count);

    if (m->pixels) {
	for (i = 0; i < m->count; i++) {
	    rgb.red = rgb.green = rgb.blue = 0;
	    rgb.flags = DoRed | DoGreen | DoBlue;
	    rgb.pixel = m->pixels[i];
	    XStoreColor (display, m->cmap, &rgb);
	}
	XFreeColors (display, m->cmap, m->pixels, m->count, 0);
    }
    if (!count) {
	if (m->pixels)
	    free (m->pixels);
	return (count);
    }
    if (m->pixels) {
	m->pixels = (Pixel *) realloc ((char *) m->pixels,
				       sizeof (Pixel) * count);
    } else {
	m->pixels = (Pixel *) malloc (sizeof (Pixel) * count);
    }
    assert (m->pixels);
    m->count = count;
    while (1) {
	if (!XAllocColorCells (display, m->cmap, False, NULL, 0,
			       m->pixels, m->count)) {
	    if (m->count == 1) {
		printf ("Error allocating colors\n");
		printf ("%d %d\n", m->pixels[0], m->pixels[1]);
		exit (1);
	    }
	    m->count--;
	} else
	    break;
    }
    return (m->count);
}

int             XnHSVGridSave (grid, filename)
XnHSVGrid      *grid;
char           *filename;
{
    XnHSVColor     *corner;
    XColor         *rgb;
    FILE           *stream;
    int             i;

    if ((stream = fopen (filename, "w"))) {
	fprintf (stream, "%d %d\n", grid->rows, grid->cols);
	corner = &grid->bands[0]->beg;
	fprintf (stream, "%g %g %g\n", corner->h, corner->s, corner->v);
	corner = &grid->bands[0]->end;
	fprintf (stream, "%g %g %g\n", corner->h, corner->s, corner->v);
	corner = &grid->bands[grid->rows - 1]->beg;
	fprintf (stream, "%g %g %g\n", corner->h, corner->s, corner->v);
	corner = &grid->bands[grid->rows - 1]->end;
	fprintf (stream, "%g %g %g\n", corner->h, corner->s, corner->v);
	for (i = 0; i < grid->rows * grid->cols; i++) {
	    rgb = &grid->bands[i / grid->cols]->colors[i % grid->cols];
	    fprintf (stream, "%d %d %d %d\n",
		     i, rgb->red, rgb->green, rgb->blue);
	}
	fclose (stream);
	return (0);
    }
    return (-1);
}

XnHSVGrid      *XnHSVGridLoad (filename)
char           *filename;
{
    XnHSVGrid      *grid;
    XnHSVColor     *corner;
    int             rows, cols;
    FILE           *stream;

    if ((stream = fopen (filename, "r"))) {
	fscanf (stream, "%d %d\n", &rows, &cols);
	grid = XnNewHSVGrid (rows, cols);
	corner = &grid->bands[0]->beg;
	fscanf (stream, "%lg %lg %lg\n", &corner->h, &corner->s, &corner->v);
	corner = &grid->bands[0]->end;
	fscanf (stream, "%lg %lg %lg\n", &corner->h, &corner->s, &corner->v);
	corner = &grid->bands[grid->rows - 1]->beg;
	fscanf (stream, "%lg %lg %lg\n", &corner->h, &corner->s, &corner->v);
	corner = &grid->bands[grid->rows - 1]->end;
	fscanf (stream, "%lg %lg %lg\n", &corner->h, &corner->s, &corner->v);
	fclose (stream);
	return (grid);
    }
    return (NULL);
}

int             XnXColorIntensity (rgb)
XColor         *rgb;
{
    XnHSVColor      hsv;

    XnXColortoHSV (&hsv, rgb->red, rgb->green, rgb->blue);
    return ((int) (256 * hsv.v) << 8);
}

Colormap XnCreateColormap (display)
Display *display;
{
    Colormap cmap;
    XColor *cells;
    int screen;
    int total;
    int i;

    screen = DefaultScreen (display);

    total = DisplayCells (display, screen);
    cells = (XColor *) malloc (total * sizeof (XColor));
    for (i = 0; i < total; i++)
	cells[i].pixel = i;
    XQueryColors (display, DefaultColormap (display, screen), cells, total);
    cmap = XCreateColormap (display, DefaultRootWindow (display),
        DefaultVisual (display, screen), AllocAll);
    XStoreColors (display, cmap, cells, total);
    XInstallColormap (display, cmap);
    free (cells);
    return (cmap);
}
