/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Dictionary.c,v 1.2 1994/02/18 19:56:13 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Dictionary.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <malloc.h>

#include <Xn/Xn.h>
#include <Xm/Text.h>
#include <Xm/LabelG.h>
#include <Xn/Dictionary.h>

static XnItemAttr **item_list = (XnItemAttr **) (0);
static int      item_count = 0;

typedef struct {
    char           *title;
    int             data;
    XnDataType     type;
    XnIntAttr      soeint;
}               XnInt;

typedef struct {
    char           *title;
    int             data;
    XnDataType     type;
    XnRealAttr     soereal;
}               XnReal;

static XnInt   IntDictionary[] = {
    {"SHORT INT:", XN_SHORT_INT, XN_INT, {-32767, 32768, 0, 5}},
    {"SHORT POSITIVE INT:", XN_SHORT_POS_INT, XN_INT, {0, 32768, 0, 5}},
    {"SHORT NEGATIVE INT:", XN_SHORT_NEG_INT, XN_INT, {-32767, 0, 0, 5}},

    {"LONG INT:", XN_LONG_INT, XN_INT, {-32767, 32768, 0, 5}},
    {"LONG POSITIVE INT:", XN_LONG_POS_INT, XN_INT, {0, 32768, 0, 5}},
    {"LONG NEGATIVE INT:", XN_LONG_NEG_INT, XN_INT, {-32767, 0, 0, 5}},
    NULL
};

static XnReal   RealDictionary[] = {
    NULL
};

static short   compare_entry (a, b)
XnItemAttr **a, **b;
{
    return ((*a)->key - (*b)->key);
}

static void     init_list ()
{
    int             i;

    item_list = (XnItemAttr **) malloc (sizeof (XnItemAttr **) * 1000);
    item_count = 0;

    for (i = 0; RealDictionary[i].title; i++) {
	item_list[item_count] = (XnItemAttr *) (RealDictionary + i);
	item_count++;
    }
    for (i = 0; IntDictionary[i].title; i++) {
	item_list[item_count] = (XnItemAttr *) (IntDictionary + i);
	item_count++;
    }
    qsort (item_list, item_count, sizeof (XnItemAttr *), compare_entry);
}

XnItemAttr    *_XnItemSearch (data)
int             data;
{
    XnItemAttr    *item = (XnItemAttr *) (0);
    int             i;

    if (!item_list)
	init_list ();

    for (i = 0; i < item_count; i++) {
	if (data == item_list[i]->key) {
	    item = item_list[i];
	    break;
	}
    }
    return (item);
}

void XnPrintItems ()
{
    XnItemAttr    *attr;
    int i;

    printf ("Data Dictionary\n");

    for (i = 0; i < item_count; i++) {
	attr = item_list[i];
	printf ("%-19s%4d%4d\n", attr->title, attr->key, attr->type);
	switch (attr->type) {
	    case XN_INT:
		printf ("  %4d %4d %4d %4d\n",
					       attr->item.intattr.min,
					       attr->item.intattr.max,
					       attr->item.intattr.def,
					       attr->item.intattr.digits);
		break;
	    case XN_FLOAT:
	    case XN_DOUBLE:
		printf ("  %4g %4g %4g %4d %4d\n",
						attr->item.realattr.min,
						attr->item.realattr.max,
						attr->item.realattr.def,
						attr->item.realattr.digits,
					      attr->item.realattr.accuracy);
		break;
	    case XN_STR:
		break;
	}
    }
}

Widget            XnCreateItem (parent, data)
Widget          parent;
XnCallbackData    *data;
{
    XnItemAttr    *attr;
    Widget label;

    if ((attr = _XnItemSearch (data->data))) {
	label = XtVaCreateManagedWidget (attr->title,
					       xmLabelGadgetClass, parent,
					       NULL);
	switch (attr->type) {
	    case XN_INT:
		data->w = (Widget) XnCreateVerifiedIntText (parent,
					       attr->item.intattr.min,
					       attr->item.intattr.max,
					       attr->item.intattr.def,
					       attr->item.intattr.digits);
		break;
	    case XN_FLOAT:
	    case XN_DOUBLE:
		data->w = (Widget) XnCreateVerifiedRealText (parent,
						attr->item.realattr.min,
						attr->item.realattr.max,
						attr->item.realattr.def,
						attr->item.realattr.digits,
					      attr->item.realattr.accuracy);
		break;
	    case XN_CHAR:
		break;
	}
    }
    return (label);
}
