/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Filter.c,v 1.2 1994/02/18 20:35:54 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Filter.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <Xn/Xn.h>
#include <Xm/Text.h>

#define TextShortFilter TextIntFilter
#define TextLongFilter TextIntFilter
#define TextDoubleFilter TextFloatFilter

static int      pointCheck (char *s)
{
    while (*s)
	if (*s++ == '.')
	    return True;

    return False;
}

static char    *buildNewString (Widget w, XmTextVerifyPtr tvp, int *nlen)
{
    char           *str, *ptr, *new;
    int             len, oldlen, newlen;
    int             i, j;

    ptr = tvp->text->ptr;
    len = tvp->text->length;

    str = XmTextGetString (w);

    oldlen = strlen (str);
    newlen = oldlen + len - tvp->endPos + tvp->startPos;

    new = XtMalloc (newlen + 1);

    for (i = 0; i < tvp->startPos; i++)
	new[i] = str[i];
    for (i = tvp->startPos, j = 0; j < len; i++, j++)
	new[i] = ptr[j];
    for (i = tvp->startPos + len, j = tvp->endPos; j < oldlen; i++, j++)
	new[i] = str[j];

    new[newlen] = '\0';

    XtFree (str);

    *nlen = newlen;
    return new;
}

void            TextFloatFilter (Widget w, XtPointer client, XmTextVerifyPtr tvp)
{
    int             has_point;
    char           *str;

    int             i, j, start = 0;

    tvp->doit = True;

    if (tvp->text->length)
	if ((tvp->startPos == 0) && (tvp->text->ptr[0] == '-'))
	    start = 1;

    str = XmTextGetString (w);
    has_point = pointCheck (str);

    for (i = start; i < tvp->text->length; i++)
	if (!isdigit (tvp->text->ptr[i]))
	    if (!((tvp->text->ptr[i] == '.') && !has_point)) {
		for (j = i; j < tvp->text->length; j++)
		    if (tvp->text->ptr[j] != ' ') {
			tvp->doit = False;
			break;
		    }
	    }
    XtFree (str);
}

void            TextIntFilter (Widget w, XtPointer client, XmTextVerifyPtr tvp)
{
    int             i, j, start = 0;

    tvp->doit = True;

    if (tvp->text->length)
	if ((tvp->startPos == 0) && (tvp->text->ptr[0] == '-'))
	    start = 1;

    for (i = start; i < tvp->text->length; i++)
	if (!isdigit (tvp->text->ptr[i])) {
	    /* check for trailing blanks */
	    for (j = i; j < tvp->text->length; j++)
		if (tvp->text->ptr[j] != ' ') {
		    tvp->doit = False;
		    break;
		}
	}
}

void            TextCapsFilter (Widget w, XtPointer client, XmTextVerifyPtr tvp)
{
    int             i;

    tvp->doit = True;
    for (i = 0; i < tvp->text->length; i++)
	if (islower (tvp->text->ptr[i]))
	    tvp->text->ptr[i] = toupper (tvp->text->ptr[i]);
}

void            TextStringFilter (Widget w, XnTextFilterProc proc, XmTextVerifyPtr tvp)
{
    char           *buf;
    int             len;

    buf = buildNewString (w, tvp, &len);

    if (proc)
	tvp->doit = (*proc) (buf);

    XtFree (buf);
}
