/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: IntVal.c,v 1.3 1994/02/18 20:35:55 dta Exp $
 **
 **  $Source: /cvs/lib/xn/IntVal.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <stdlib.h>

#include <Xn/Xn.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#ifdef DEBUG
static void     XnPrintInt (attr)
XnIntAttr     *attr;
{
    printf ("XnIntAttr %d %d %d %d\n",
	    attr->min,
	    attr->max,
	    attr->def,
	    attr->digits);
}

#endif

static int      VerifyInt (text, attr)
char           *text;
XnIntAttr     *attr;
{
    int             val;
    int             len;

    if (!text || !*text) {
	val = attr->def;
    } else {
	sscanf (text, "%d%n", &val, &len);
	if (!len) {
	    val = attr->def;
	} else if (val < attr->min) {
	    val = attr->min;
	} else if (val > attr->max) {
	    val = attr->max;
	}
    }
    return (val);
}

static XtCallbackProc IntModifyVerify (w, attr, cb)
Widget          w;
XnIntAttr     *attr;
XmTextVerifyCallbackStruct *cb;
{
    char           *ptr;
    int             len;
    int             i = 0;

#ifdef DEBUG
    printf ("IntModifyVerify %d %d\t", cb->reason, cb->text->length);
    XnPrintInt (attr);
#endif

    ptr = cb->text->ptr;
    len = cb->text->length;

    if (len) {
	if (*ptr == '-') {
	    ptr++;
	    len--;
	}
	if (len <= attr->digits) {
	    for (i = 0; i < len; i++) {
		if (!isdigit (ptr[i])) {
		    break;
		}
	    }
	}
    }
    if (i != len)
	cb->doit = False;

    return (0);
}

static XtCallbackProc IntValueChanged (w, attr, cb)
Widget          w;
XnIntAttr     *attr;
XmAnyCallbackStruct *cb;
{

#ifdef DEBUG
    printf ("IntValueChanged %d\t", cb->reason);
    XnPrintInt (attr);
#endif

    return (0);
}

static XtCallbackProc IntLosingFocus (w, attr, cb)
Widget          w;
XnIntAttr     *attr;
XmAnyCallbackStruct *cb;
{
    char           *text, str[256];
    int             val;

#ifdef DEBUG
    printf ("IntLosingFocus %d\t", cb->reason);
    XnPrintInt (attr);
#endif

    if ((text = XmTextGetString (w))) {
	val = VerifyInt (text, attr);
	sprintf (str, "%d", val);
	if (strcmp (str, text))
	    XmTextSetString (w, str);
	XtFree (text);
    }
    return (0);
}

static XtCallbackProc IntHelp (w, attr, cb)
Widget          w;
XnIntAttr     *attr;
XmAnyCallbackStruct *cb;
{

#ifdef DEBUG
    printf ("IntHelp %d\t", cb->reason);
    XnPrintInt (attr);
#endif

    return (0);
}

static XtCallbackProc IntDestroy (w, attr, cb)
Widget          w;
XnIntAttr     *attr;
XmAnyCallbackStruct *cb;
{

#ifdef DEBUG
    printf ("IntDestroy %d\t", cb->reason);
    XnPrintInt (attr);
#endif

    XtFree ((XtPointer) attr);
    return (0);
}

/*
 * public functions
 */

void XnIntAddCallbacks (w, attr)
Widget w;
XnIntAttr     *attr;
{
    XnAddCallback (w, XmNmodifyVerifyCallback, IntModifyVerify, attr);
    XnAddCallback (w, XmNvalueChangedCallback, IntValueChanged, attr);
    XnAddCallback (w, XmNlosingFocusCallback, IntLosingFocus, attr);
    XnAddCallback (w, XmNdestroyCallback, IntDestroy, attr);
    XnAddCallback (w, XmNhelpCallback, IntHelp, attr);
}

Widget          XnCreateVerifiedIntText (parent, min, max, def, digits)
Widget          parent;
int             min;
int             max;
int             def;
int             digits;
{
    char            value[256];
    XnIntAttr     *attr;
    Widget          text;

    attr = (XnIntAttr *) XtMalloc (sizeof (XnIntAttr));

    attr->min = min;
    attr->max = max;
    attr->def = def;
    attr->digits = digits;

    sprintf (value, "%d", def);

    text = XtVaCreateManagedWidget ("IntText",
				    xmTextFieldWidgetClass, parent,
				    XmNmaxLength, digits,
				    XmNcolumns, digits,
				    XmNvalue, value,
				    NULL);
    XnIntAddCallbacks (text, attr);

    return (text);
}
