/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Menu.c,v 1.7 1994/02/18 19:56:17 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Menu.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Separator.h>
#include <Xm/SeparatoG.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/RowColumn.h>

#include "Xn.h"

static void beg_cb (Widget w, XtPointer data, XtPointer cb)
{
    XnStartWorking ();
}

static void end_cb (Widget w, XtPointer data, XtPointer cb)
{
    XnStopWorking ();
}

static char *menustr (str, name, mnemonic)
char *str;
char *name;
int *mnemonic;
{
    char *ptr = name;

    *mnemonic = -1;
    while (*ptr) {
	if (*ptr == '&')
	    *mnemonic = ptr++ - name;
	else
	    *str++ = *ptr++;
    }
    *str = NULL;
    return (str);
}

static Widget initmenu (parent, menudata, depth)
Widget parent;
XnMenuData *menudata;
int depth;
{
    char str [32];
    Widget menu;
    XmString label;
    int mnemonic;
    int i, j;

    if (!depth) {
        for (i = 0; menudata [i].cbd.name; i++) {
	    menu = XmCreatePulldownMenu (parent, "pulldown", NULL, 0);
	    menustr (str, menudata [i].cbd.name, &mnemonic);
	    label = XmStringCreateSimple (str);
	    menudata [i].cbd.w = XtVaCreateManagedWidget (str,
		xmCascadeButtonWidgetClass, parent,
		XmNlabelString, label,
		XmNsubMenuId, menu,
		XmNuserData, i,
		NULL);
	    if (mnemonic >= 0)
		XtVaSetValues (menudata [i].cbd.w,
		    XmNmnemonic, str[mnemonic],
		    NULL);
	    XmStringFree (label);
	    if (menudata[i].cbd.data)
		initmenu (menu, menudata [i].cbd.data, depth + 1);
	}
    } else {
	for (j = i = 0; menudata [i].cbd.name; i++) {
	    if (!*menudata [i].cbd.name) {
		menudata [i].cbd.w = XtVaCreateManagedWidget ("separator",
		    xmSeparatorWidgetClass, parent,
		    NULL);
	    } else if (!menudata [i].cbd.proc && menudata [i].cbd.data) {
		/*
		 * more sub menus
		 */
	    } else {
		menustr (str, menudata [i].cbd.name, &mnemonic);
		menudata [i].cbd.w = XtVaCreateManagedWidget (str,
		    xmPushButtonWidgetClass, parent,
		    XmNuserData, i,
		    NULL);
		if (mnemonic >= 0)
		    XtVaSetValues (menudata [i].cbd.w,
			XmNmnemonic, str[mnemonic],
			NULL);
		if (menudata [i].accelerator) {
		   label = XmStringCreateSimple (menudata [i].accel_text);
		   XtVaSetValues (menudata [i].cbd.w,
		       XmNaccelerator, menudata [i].accelerator,
		       XmNacceleratorText, label,
		       NULL);
		    XmStringFree (label);
		}
		XnAddCallback (menudata[i].cbd.w, XmNactivateCallback, beg_cb, 0);
		XnAddCallback (menudata[i].cbd.w, XmNactivateCallback,
		    menudata[i].cbd.proc, menudata[i].cbd.data);
		XnAddCallback (menudata[i].cbd.w, XmNactivateCallback, end_cb, 0);
		menudata [i].cbd.data = (XtPointer)(j++);
	    }
	}
    }
    return (parent);
}

Widget XnCreateMenus (parent, menudata)
Widget parent;
XnMenuData *menudata;
{
    Widget menubar;

    menubar = XmCreateMenuBar (parent, "menubar", NULL, 0);
    initmenu (menubar, menudata, 0);
    XtManageChild (menubar);

    if (XmIsForm (parent)) {
	XtVaSetValues (menubar,
	    XmNtopAttachment, XmATTACH_FORM,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNrightAttachment, XmATTACH_FORM,
	    NULL);
    }

    return (menubar);
}

Widget XnMenuAddFocus (menudata, proc)
XnMenuData *menudata;
XtCallbackProc proc;
{
    int i;

    for (i = 0; menudata [i].cbd.name; i++) {
	if (XmIsCascadeButton (menudata [i].cbd.w)) {
	    XtAddCallback (menudata[i].cbd.w, XmNcascadingCallback, proc, menudata + i);
	} else if (XmIsPushButton (menudata [i].cbd.w)) {
	    XtAddCallback (menudata[i].cbd.w, XmNarmCallback, proc, menudata + i);
	}
	if (!menudata [i].cbd.proc && menudata [i].cbd.data)
	    XnMenuAddFocus (menudata [i].cbd.data, proc);
    }
}

Widget XnMenuSetSensitive (menudata, mask)
XnMenuData *menudata;
int mask;
{
    int i;

    for (i = 0; menudata [i].cbd.name; i++) {
	if (XmIsPushButton (menudata [i].cbd.w))
	    XtSetSensitive (menudata[i].cbd.w,
			    ((menudata [i].mask & mask) != 0));
	if (!menudata [i].cbd.proc && menudata [i].cbd.data)
	    XnMenuSetSensitive (menudata [i].cbd.data, mask);
    }
}
