/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Message.c,v 1.2 1994/02/18 19:56:18 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Message.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdarg.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/MessageB.h>

#include "Xn.h"

static void cancel_cb (Widget window, XtPointer data, XtPointer cbd)
{
    XnPopdown (window);
}

static void doit (Widget child, XtCallbackProc proc, XtPointer data)
{
    if (proc)
	XnAddCallback (child, XmNactivateCallback, proc, data);
    else
	XtUnmanageChild (child);
}

static void initialize (Widget w,
	XtCallbackProc ok, XtCallbackProc cancel, XtCallbackProc help, XtPointer data)
{
    doit (XmMessageBoxGetChild (w, XmDIALOG_OK_BUTTON), ok, data);

    if (!ok)
	XnWidgetPutStr (XmMessageBoxGetChild (w, XmDIALOG_CANCEL_BUTTON), "Close");

    if (!cancel)
	cancel = cancel_cb;

    doit (XmMessageBoxGetChild (w, XmDIALOG_CANCEL_BUTTON), cancel, data);
    doit (XmMessageBoxGetChild (w, XmDIALOG_HELP_BUTTON), help, data);
}

Widget          XnCreateFooter (Widget parent,
	XtCallbackProc ok, XtCallbackProc cancel, XtCallbackProc help, XtPointer data)
{
    Widget footer = XtVaCreateManagedWidget ("footer",
				      xmMessageBoxWidgetClass, parent,
				      NULL);

    if (XmIsForm (parent)) {
	XtVaSetValues (footer,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNrightAttachment, XmATTACH_FORM,
	    XmNbottomAttachment, XmATTACH_FORM,
	    NULL);
    }

    XtUnmanageChild (XmMessageBoxGetChild (footer, XmDIALOG_MESSAGE_LABEL));

    initialize (footer, ok, cancel, help, data);

    return (footer);
}

static Widget error_dialog = NULL;

void XnKillErrorMessage (void)
{
    if (error_dialog && XtIsManaged (error_dialog))
	XtUnmanageChild (error_dialog);
}

void XnErrorMessage (const char *fmt, ...)
{
    extern Widget xntop;
    char buffer [1024];
    va_list ap;

    va_start (ap, fmt);
    vsprintf (buffer, fmt, ap);
    va_end (ap);
    
    if (!error_dialog) {
	error_dialog = XmCreateErrorDialog (xntop, "error", NULL, 0);

	XnWidgetPutStr (XtParent (error_dialog), "Error Message");
	XnSetNoResize (XtParent (error_dialog));

	initialize (error_dialog, NULL, NULL, NULL, NULL);
    }

    XnWidgetPrintf (error_dialog, buffer);
    XtManageChild (error_dialog);

    XtPopup (XtParent (error_dialog), XtGrabNone);
}


static Widget info_dialog = NULL;

void XnKillInfoMessage (void)
{
    if (info_dialog && XtIsManaged (info_dialog))
	XtUnmanageChild (info_dialog);
}

void XnInfoMessage (const char *title, const char *fmt, ...)
{
    extern Widget xntop;
    char buffer [1024];
    va_list ap;

    va_start (ap, fmt);
    vsprintf (buffer, fmt, ap);
    va_end (ap);
    
    if (!info_dialog) {
	info_dialog = XmCreateInformationDialog (xntop, "info", NULL, 0);

	XnWidgetPutStr (XtParent (info_dialog), title);
	XnSetNoResize (XtParent (info_dialog));

	initialize (info_dialog, NULL, NULL, NULL, NULL);
    }

    XnWidgetPrintf (info_dialog, buffer);
    XtManageChild (info_dialog);

    XtPopup (XtParent (info_dialog), XtGrabNone);
}
