/*******************************<+>***************************
 **                             DTA
 *************************************************************
 **
 **  $Id: XColor.c,v 1.3 1994/02/18 02:55:40 dta Exp $
 **
 **  $Source: /cvs/lib/xn/XColor.c,v $
 **
 **  What @(#):
 **
 **  Author: Dale T. Anderson
 **
 *******************************<+>***************************/
 
/*******************************<+>***************************
 **
 **  $Id: XColor.c,v 1.3 1994/02/18 02:55:40 dta Exp $
 **
 **  $Source: /cvs/lib/xn/XColor.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Authors:         Dale T. Anderson
 **                   Robert F. Hastings
 **
 *******************************<+>***************************/

#include <math.h>
#include <assert.h>
#include <Xm/Xm.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>

#include "Xn.h"

/* RFH: for convert args: WidgetRec */
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>


#define	done(type, value) \
	{							\
	    if (toVal->addr != NULL) {				\
		if (toVal->size < sizeof(type)) {		\
		    toVal->size = sizeof(type);			\
		    return False;				\
		}						\
		*(type*)(toVal->addr) = (value);		\
	    }							\
	    else {						\
		static type static_val;				\
		static_val = (value);				\
		toVal->addr = (XtPointer)&static_val;		\
	    }							\
	    toVal->size = sizeof(type);				\
	    return True;					\
	}

XtConvertArgRec XnColorConvertArgs[] = {
    {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.screen),
     sizeof(Screen *)},
    {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.colormap),
     sizeof(Colormap)}
};

/*
 * RFH: partially taken from X11R5 Intrinsic code.
 */
Boolean XnCvtStringToPixel (dpy, args, num_args, fromVal, toVal, closure_ret)
Display        *dpy;
XrmValuePtr	args;
Cardinal       *num_args;
XrmValuePtr	fromVal;
XrmValuePtr	toVal;
XtPointer      *closure_ret;	/* Xt uses this when freeing the color, lazy us we don't free */
{
    String	str = (String)fromVal->addr;
    XColor	screenColor;
    XColor	exactColor;
    Screen     *screen;
    Colormap	colormap;
    String	params[1];
    Cardinal	num_params=1;
    Status	status;

    if (*num_args != 2) {
	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
			"wrongParameters",
			"cvtStringToPixel",
			"XnConverterError",
			"String to pixel conversion needs a colormap argument",
			(String *)NULL, (Cardinal *)NULL);
	return False;
    }

    screen = *((Screen **) args[0].addr);
    colormap = *((Colormap *) args[1].addr);

    status = XAllocNamedColor(dpy, colormap, (char *)str, &screenColor, &exactColor);

    if (status) {
	/* We're happy */
	done(Pixel, screenColor.pixel);
    }

    /*
     * Give warning if the color name is bad, otherwise return BlackPixel.
     * Xt would also give warning even if the alloc failed.
     */
    if (XLookupColor(dpy, colormap, (char *)str,
		     &exactColor, &screenColor) == 0) {
	params[0] = str;
	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
			"badValue",
			"cvtStringToPixel",
			"XnConverterError",
			"Color name \"%s\" is not defined",
			params, &num_params);
	/*
	 * Bummer: don't record this in the xn_color_hist
	 */
	return False;
    }

    /*
     * Lie: Well we did not get the pixel but give no warning.
     * return black.  Works for the bg cause the fg is white (which should never fail).
     */
    done(Pixel, BlackPixelOfScreen(screen));
}

static Boolean IsPixelAlreadyAllocated (color_reqs, num_reqs, pixel)
XnColorRequest  *color_reqs;
Cardinal	num_reqs;
Pixel		pixel;
{
    Cardinal	i;
    
    for (i = 0; i < num_reqs; i++) {
	if (color_reqs[i].allocated == True &&
	    *color_reqs[i].addr == pixel)
	    return True;
    }
    return False;
}

/*
 * Dist in 3space.  For RGB colors there may exist
 * a more appropriate way to estimate color closeness.
 */
static double XColorDistance (xc1, xc2)
XColor	*xc1;
XColor	*xc2;
{
    double r, g, b;

    r = xc1->red   - xc2->red;
    g = xc1->green - xc2->green;
    b = xc1->blue  - xc2->blue;
    return ((r*r) + (g*g) + (b*b));
}

/*
 * When used with qsort this compare will
 * place the colors with highest priority at the beginning.
 * If two colors have the same priority then the one with
 * the largest min_dist is placed first.
 */
static int color_reqs_cmp(cr1, cr2)
XnColorRequest *cr1;
XnColorRequest *cr2;
{
    if (cr1->priority < cr2->priority)
	return 1;
    if (cr1->priority > cr2->priority)
	return -1;
    /*
     * Priorities the same.
     */
    if (cr1->min_dist < cr2->min_dist)
	return 1;
    if (cr1->min_dist > cr2->min_dist)
	return -1;
    return 0;
}

/*
 * RFH: could make this public to free colors.  Need to set allocated to true
 * in final allocation loop in XnRequestColors
 */
static XnFreeAllocatedRequests(top, color_reqs, num_reqs)
Widget 		top;
XnColorRequest *color_reqs;
Cardinal	num_reqs;
{
    Display    *display = XtDisplay(top);
    Colormap	cmap;
    Cardinal	i, j;
    Pixel      *allocated_pixels;
    Cardinal	next_pixel;

    XtVaGetValues(top, XmNcolormap, &cmap, NULL);

    allocated_pixels = (Pixel *)XtMalloc(sizeof (Pixel) * num_reqs);
    next_pixel = 0;

    for (i = 0; i < num_reqs; i++) {
	if (!color_reqs[i].allocated)
	    continue;
	for (j = 0; j < next_pixel; j++)
	    if (allocated_pixels[j] == *color_reqs[i].addr)
		break;		/* already in allocated_pixels */
	if (j == next_pixel) {
	    /* add to allocated_pixels */
	    allocated_pixels[j] = *color_reqs[i].addr;
	    next_pixel++;
	}
    }
    XFreeColors(display, cmap, allocated_pixels, next_pixel, 0L);
}

void XnRequestColors(top, color_reqs, num_reqs)
Widget		top;
XnColorRequest *color_reqs;
Cardinal	num_reqs;
{
    Display    *display = XtDisplay(top);
    int 	screen = DefaultScreen(display);
    Pixel	black = BlackPixel(display, screen);
    Pixel	white = WhitePixel(display, screen);
    Colormap	cmap;
    Cardinal	num_cells;
    XColor     *pallette;
    XColor	cc_def, rgb_db_def, hw_def;
    Cardinal	i;
    Pixel	j;
    Boolean	failed = False;

    XtVaGetValues(top, XmNcolormap, &cmap, NULL);
    for (i = 0; i < num_reqs; i++) {
	/*
	 * Convert name to a RBG, if we can.  If we can't then warn user.
	 * RFH: should we support XtDefaultForeground and XtDefaultBackground?
	 */
	if (color_reqs[i].color_name && color_reqs[i].color_name[0]) {
	    if (XLookupColor(display, cmap, color_reqs[i].color_name, &rgb_db_def, &hw_def)) {
		/*
		 * Copy RGB for later.
		 */
		color_reqs[i].red   = hw_def.red;
		color_reqs[i].green = hw_def.green;
		color_reqs[i].blue  = hw_def.blue;
	    } else {
		String		params[1];
		Cardinal	num_params;
		
		/*
		 * Bricked on lookup color (no such color name).
		 * Fake like we are in a Converter routine and warn user.
		 */
		params[0] = color_reqs[i].color_name;
		num_params = 1;
		XtAppWarningMsg(XtDisplayToApplicationContext(display),
				"badValue",
				"cvtStringToPixel",
				"XnConverterError",
				"Color name \"%s\" is not defined",
				params, &num_params);
	    }
	}
	/*
	 * The color request NOW has a RGB tripple, try to allocate it
	 */
	cc_def.red   = color_reqs[i].red;
	cc_def.green = color_reqs[i].green;
	cc_def.blue  = color_reqs[i].blue;
#ifdef DEBUG_COLOR
	printf("%x: %6d %6d %6d\n", color_reqs[i].addr, cc_def.red, cc_def.green, cc_def.blue);
#endif
	if (XAllocColor(display, cmap, &cc_def)) {
	    color_reqs[i].allocated = True;
	    *color_reqs[i].addr = cc_def.pixel;
	} else {
	    color_reqs[i].allocated = False;
	    failed = True;
	}
    }
    /*
     * All colors allocated.
     */
    if (!failed)
	return;
    /*
     * Since some allocations failed try to match the colors to those already allocated.
     * All cells MUST be allocated because, at least one of our requests failed.
     * Of course there is a race condition because, some other client may free a 
     * cell while we are matching.
     */
    num_cells = DisplayCells(display, screen);
    pallette = (XColor *)XtMalloc(sizeof(XColor) * num_cells);
    /*
     * Get all of the cells
     */
    for (i = 0; i < num_cells; i++) {
	pallette[i].flags = DoRed | DoGreen | DoBlue;
	pallette[i].pixel = i;
    }
    XQueryColors(display, cmap, pallette, num_cells);
#ifdef DEBUG_COLOR
    /*
     * Print all of the cells
     */
    for (i = 0; i < num_cells; i++)
	printf("%2d:%2d %6d %6d %6d\n",
	       i,
	       pallette [i].pixel,
	       pallette [i].red,
	       pallette [i].green,
	       pallette [i].blue);
#endif
    /*
     * foreach color request
     *	foreach color
     *	 find closest color.
     */
    for (i = 0; i < num_reqs; i++) {
	double min_dist = -1.0;
	double dist;
	double dist_black;
	Pixel  closest = white;

	hw_def.red   = color_reqs[i].red;
	hw_def.green = color_reqs[i].green;
	hw_def.blue  = color_reqs[i].blue;
	for (j = 0; j < num_cells; j++) {
	    if (IsPixelAlreadyAllocated(color_reqs, num_reqs, j))
		continue;
	    dist =       XColorDistance(&pallette[j], &hw_def);
	    dist_black = XColorDistance(&pallette[j], &pallette[black]);
	    if (dist_black <= dist && dist_black != 0.0)
		continue;
	    if (min_dist > dist || min_dist < 0.0) {
		min_dist = dist;
		closest = j;
	    }
	}
	color_reqs[i].min_dist = min_dist;
	color_reqs[i].closest  = closest;
    }
    /*
     * Free all the colors we already allocated.
     */
    XnFreeAllocatedRequests(top, color_reqs, num_reqs);
    /*
     * Now we have every color's min_dist.  Find which pixel is the furthest away, and try
     * to allocate it first.  Whenever an allocate fails use the closest pixel we
     * found ealier.
     *
     * Use qsort to order the reqs by priority and distance.
     */
    qsort(color_reqs, num_reqs, sizeof (XnColorRequest), color_reqs_cmp);
    for (i = 0; i < num_reqs; i++) {
	cc_def.red   = color_reqs[i].red;
	cc_def.green = color_reqs[i].green;
	cc_def.blue  = color_reqs[i].blue;
	if (XAllocColor(display, cmap, &cc_def))
	    *color_reqs[i].addr = cc_def.pixel;
	else
	    *color_reqs[i].addr = color_reqs[i].closest;
    }
#ifdef DEBUG_COLOR
    for (i = 0; i < num_reqs; i++)
	printf("%X: %2d %6d %6d %6d\n",
	       color_reqs[i].addr, *color_reqs[i].addr,
	       pallette[i].red, pallette[i].green, pallette[i].blue);
#endif
}
