/*
 * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
 *                        All rights reserved
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of Bellcore not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * Bellcore.
 *
 * BELLCORE MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL BELLCORE OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
 * ING TO THE SOFTWARE.
 *
 * MatrixWidget Author: Andrew Wason, Bellcore, aw@n.bellcore.com
 */

#ifndef _Xn_Matrix_h
#define _Xn_Matrix_h

/*
 *  "@(#)Matrix.h	3.8 7/8/92"
 */

/*
 * Matrix Widget public include file
 */

#include <Xm/Xm.h>
#include <X11/Core.h>


/* Resources:
 * Name			Class			RepType		Default Value
 * ----			-----			-------		-------------
 * boldLabels		BoldLabels		Boolean		False
 * cellHighlightThickness HighlightThickness	HorizontalDimension 2
 * cellMarginHeight	MarginHeight		VerticalDimension   5
 * cellMarginWidth	MarginWidth		HorizontalDimension 5
 * cells		Cells			StringTable	NULL
 * cellShadowThickness	ShadowThickness		HorizontalDimension 2
 * colors		Colors			PixelTable	NULL
 * columnAlignments	Alignments		AlignmentArray	dynamic
 * columnLabelAlignmentsAlignments		AlignmentArray	dynamic
 * columnLabels		Labels			StringArray	NULL
 * columnMaxLengths	ColumnMaxLengths	MaxLengthArray	NULL
 * columnWidths		ColumnWidths		WidthArray	NULL
 * columns		Columns			int		0
 * enterCellCallback	Callback		Callback	NULL
 * fixedColumns		FixedColumns		Dimension	0
 * fixedRows		FixedRows		Dimension	0
 * fontList		FontList		FontList	fixed
 * leaveCellCallback	Callback		Callback	NULL
 * modifyVerifyCallback	Callback		Callback	NULL
 * rowLabelAlignment	Alignment		Alignment	XmALIGNMENT_END
 * rowLabelWidth	RowLabelWidth		Short		dynamic
 * rowLabels		Labels			StringArray	NULL
 * rows			Rows			int		0
 * selectCellCallback	Callback		Callback	NULL
 * selectedCells	SelectedCells		BooleanTable	dynamic
 * space		Space			Dimension	6
 * textTranslations	Translations		TranslationTable    dynamic
 * topRow		TopRow			int		0
 * traverseCellCallback	Callback		Callback	NULL
 * visibleColumns	VisibleColumns		Dimension	0
 * visibleRows		VisibleRows		Dimension	0
 */

#define XmNboldLabels "boldLabels"
#define XmNcellHighlightThickness "cellHighlightThickness"
#define XmNcellMarginHeight "cellMarginHeight"
#define XmNcellMarginWidth "cellMarginWidth"
#define XmNcellShadowThickness "cellShadowThickness"
#define XmNcells "cells"
#define XmNcolors "colors"
#define XmNcolumnAlignments "columnAlignments"
#define XmNcolumnLabelAlignments "columnLabelAlignments"
#define XmNcolumnLabels "columnLabels"
#define XmNcolumnMaxLengths "columnMaxLengths"
#define XmNcolumnWidths "columnWidths"
#define XmNeditVerifyCallback "editVerifyCallback"
#define XmNenterCellCallback "enterCellCallback"
#define XmNfixedColumns "fixedColumns"
#define XmNfixedRows "fixedRows"
#define XmNleaveCellCallback "leaveCellCallback"
#define XmNrowLabelAlignment "rowLabelAlignment"
#define XmNrowLabelWidth "rowLabelWidth"
#define XmNrowLabels "rowLabels"
#define XmNselectedCells "selectedCells"
#define XmNselectCellCallback "selectCellCallback"
/* dta */
#define XmNselectLabelCallback "selectLabelCallback"
/* dta */
#define XmNtopRow "topRow"
#define XmNtraverseCellCallback "traverseCellCallback"
#define XmNvisibleColumns "visibleColumns"
#define XmNvisibleRows "visibleRows"

#define XmCAlignments "Alignments"
#define XmCBoldLabels "BoldLabels"
#define XmCCells "Cells"
#define XmCColors "Colors"
#define XmCColumnMaxLengths "ColumnMaxLengths"
#define XmCColumnWidths "ColumnWidths"
#define XmCFixedColumns "FixedColumns"
#define XmCFixedRows "FixedRows"
#define XmCLabels "Labels"
#define XmCRowLabelWidth "RowLabelWidth"
#define XmCSelectedCells "SelectedCells"
#define XmCTopRow "TopRow"
#define XmCVisibleColumns "VisibleColumns"
#define XmCVisibleRows "VisibleRows"

#ifndef XmRStringArray
#define XmRStringArray "StringArray"
#endif
#define XmRWidthArray "WidthArray"
#define XmRMaxLengthArray "MaxLengthArray"
#define XmRAlignmentArray "AlignmentArray"
#define XmRPixelTable "PixelTable"
#define XmRBooleanTable "BooleanTable"


/* Class record constants */

extern WidgetClass xnMatrixWidgetClass;

typedef struct _XnMatrixClassRec *XnMatrixWidgetClass;
typedef struct _XnMatrixRec *XnMatrixWidget;

/*
 * External interfaces to class methods
 */

#if defined (__cplusplus) || defined(c_plusplus)
#define CONST const
extern "C" {
#else
#define CONST
#endif

extern void XnMatrixSetCell(
#if NeedFunctionPrototypes
			      Widget		/* w */,
			      int		/* row */,
			      int		/* column */,
			      CONST String	/* value */
#endif
			      );

extern void XnMatrixEditCell(
#if NeedFunctionPrototypes
			       Widget	/* w */,
			       int	/* row */,
			       int	/* column */
#endif
			       );

extern void XnMatrixSelectCell(
#if NeedFunctionPrototypes
				 Widget	/* w */,
				 int	/* row */,
				 int	/* column */
#endif
				 );

extern void XnMatrixSelectRow(
#if NeedFunctionPrototypes
				Widget	/* w */,
				int	/* row */
#endif
				);

extern void XnMatrixSelectColumn(
#if NeedFunctionPrototypes
				   Widget	/* w */,
				   int		/* column */
#endif
				 );

extern void XnMatrixDeselectAll(
#if NeedFunctionPrototypes
				 Widget	/* w */
#endif
				 );

extern void XnMatrixDeselectCell(
#if NeedFunctionPrototypes
				   Widget/* w */,
				   int	/* row */,
				   int	/* column */
#endif
				 );

extern void XnMatrixDeselectRow(
#if NeedFunctionPrototypes
				   Widget/* w */,
				   int	/* row */
#endif
				 );

extern void XnMatrixDeselectColumn(
#if NeedFunctionPrototypes
				   Widget/* w */,
				   int	/* column */
#endif
				 );

extern String XnMatrixGetCell(
#if NeedFunctionPrototypes
				Widget	/* w */,
				int	/* row */,
				int	/* column */
#endif
				);

extern Boolean XnMatrixCommitEdit(
#if NeedFunctionPrototypes2
				    Widget	/* w */,
				    Boolean	/* unmap */
#endif
				    );

extern void XnMatrixCancelEdit(
#if NeedFunctionPrototypes2
				 Widget		/* w */,
				 Boolean	/* unmap */
#endif
				 );


extern void XnMatrixAddRows(
#if NeedFunctionPrototypes
			      Widget	/* w */,
			      int	/* position */,
			      String *	/* rows */,
			      String *	/* labels */,
			      Pixel *	/* colors */,
			      int	/* num_colors */
#endif
			      );

extern void XnMatrixDeleteRows(
#if NeedFunctionPrototypes
				 Widget	/* w */,
				 int	/* position */,
				 int	/* num_rows */
#endif
				 );

extern void XnMatrixAddColumns(
#if NeedFunctionPrototypes
				 Widget		/* w */,
				 int		/* position */,
				 String *	/* columns */,
				 String *	/* labels */,
				 short *	/* widths */,
				 int *		/* max_lengths */,
				 unsigned char * /* alignments */,
				 unsigned char * /* label_alignments */,
				 Pixel *	/* colors */,
				 int		/* num_columns */
#endif
				 );

extern void XnMatrixDeleteColumns(
#if NeedFunctionPrototypes
				    Widget	/* w */,
				    int		/* position */,
				    int		/* num_columns */
#endif
				    );

extern void XnMatrixSetRowColors(
#if NeedFunctionPrototypes
				   Widget	/* w */,
				   int		/* position */,
				   Pixel *	/* colors */,
				   int		/* num_colors */
#endif
				   );

extern void XnMatrixSetColumnColors(
#if NeedFunctionPrototypes
				      Widget	/* w */,
				      int	/* position */,
				      Pixel *	/* colors */,
				      int	/* num_colors */
#endif
				      );

extern void XnMatrixSetCellColor(
#if NeedFunctionPrototypes
				      Widget	/* w */,
				      int	/* row */,
				      int	/* column */,
				      Pixel	/* color */
#endif
				      );

#if defined (__cplusplus) || defined(c_plusplus)
}
#endif
#undef CONST

/*
 * Callback reasons.  Try to stay out of range of the Motif XmCR_* reasons.
 */
typedef enum _XnReasonType {
    XnModifyVerifyReason = 102,
    XnEnterCellReason,
    XnLeaveCellReason,
    XnTraverseCellReason,
    XnSelectCellReason,
    /* dta */
    XnSelectLabelReason,
    /* dta */
} XnReasonType;

/*
 * Struct passed to modifyVerifyCallback
 */
typedef struct _XnMatrixModifyVerifyCallbackStruct {
    XnReasonType reason;
    int row, column;
    XmTextVerifyCallbackStruct *verify;
} XnMatrixModifyVerifyCallbackStruct;

/*
 * Struct passed to enterCellCallback
 */
typedef struct _XnMatrixEnterCellCallbackStruct {
    XnReasonType reason;
    int row, column;
    Boolean doit;
} XnMatrixEnterCellCallbackStruct;

/*
 * Struct passed to leaveCellCallback
 */
typedef struct _XnMatrixLeaveCellCallbackStruct {
    XnReasonType reason;
    int row, column;
    String value;
    Boolean doit;
} XnMatrixLeaveCellCallbackStruct;

/*
 * Struct passed to traverseCellCallback
 */
typedef struct _XnMatrixTraverseCellCallbackStruct {
    XnReasonType reason;
    int row, column;
    int next_row, next_column;
    int fixed_rows, fixed_columns;
    int num_rows, num_columns;
    String param;
    XrmQuark qparam;
} XnMatrixTraverseCellCallbackStruct;

/*
 * Struct passed to selectCellCallback
 */
typedef struct _XnMatrixSelectCellCallbackStruct {
    XnReasonType reason;
    int row, column;
    Boolean **selected_cells;
    String **cells;
    Cardinal num_params;
    String *params;
    XEvent *event;
} XnMatrixSelectCellCallbackStruct;

/* dta */
typedef struct _XnMatrixSelectLabelCallbackStruct {
    XnReasonType reason;
    int row, column;
    XEvent *event;
} XnMatrixSelectLabelCallbackStruct;
/* dta */

#endif /* _Xn_Matrix_h */
/* DON'T ADD STUFF AFTER THIS #endif */

